@@ -1,6 +1,15 @@
 #ifndef __DATATYPES_H
 #define __DATATYPES_H
 
+#define FREE 0
+#define LOCKED 1
+
+#define KEEP_HISTORY 1
+#define KILL_HISTORY 2
+
+#define SOFT_HYPHEN 0xAD
+#define KEY_NOTHING 99999
+
 #define HIGHLIGHT 1000
 
 typedef struct
@@ -32,6 +41,7 @@ typedef struct
     char **file;		/* array of history of files, associated with given nodes */
     int *pos;			/* history of pos offsets in viewed nodes */
     int *cursor;		/* history of cursor offsets in viewed nodes */
+    int *menu;			/* history of menu positions (in sequential reading) in viewed nodes */
   }
 InfoHistory;
 
@@ -53,6 +63,8 @@ typedef struct
   }
 HyperObject;
 
+extern int verbose;
+
 extern char *httpviewer;	/* name of http viewer (i.e. lynx) */
 extern char *ftpviewer;		/* name of ftp viewer */
 extern char *maileditor;	/* name of maileditor */
@@ -80,6 +92,7 @@ extern int maxx, maxy;		/* maximum dimensions of screen */
 extern InfoHistory infohistory;
 extern int npos;		/* position to by set when moving via history */
 extern int ncursor;		/* cursor pos to be set when..... as above */
+extern int nmenu;		/* sequential reading menu pos..... as above */
 extern int use_apropos;		/* determines if the apropos should be called
 				   if searching for aproprimate document fails */
 extern int CutManHeaders;	/* determines if man handling routines should
@@ -107,7 +120,7 @@ extern int quote_ignored;	/* quote ignored macros when watching page */
 
 void inithistory ();		/* initialize history (see struct above) 
 				 * variables for `lastread' history */
-void addinfohistory (char *file, char *node, int cursor, int pos);
+void addinfohistory (char *file, char *node, int cursor, int menu, int pos);
 				/* adds a history entry to the info file
 				 * `lastread' history */
 void dellastinfohistory ();	/* deletes last history entry */
