@@ -1,12 +1,12 @@
 #include "common_includes.h"
 
-RCSID("$Id: filehandling_functions.c,v 1.27 2000/01/27 11:38:15 misiek Exp $")
+RCSID ("$Id: filehandling_functions.c,v 1.28 2000/06/07 11:27:11 pborys Exp $")
 
-typedef struct
-  {
-    char *suffix;
-    char *command;
-  }
+     typedef struct
+       {
+	 char *suffix;
+	 char *command;
+       }
 Suffixes;
 
 /******************************************************************************
@@ -18,20 +18,20 @@ Suffixes;
 
 #define SuffixesNumber 4
 
-Suffixes suffixes[SuffixesNumber] =
-{
-  {"", "cat"},
-  {".gz", "gzip -d -q -c"},
-  {".Z", "gzip -d -q -c"},
-  {".bz2", "bzip2 -d -c"}};
+     Suffixes suffixes[SuffixesNumber] =
+     {
+       {"", "cat"},
+       {".gz", "gzip -d -q -c"},
+       {".Z", "gzip -d -q -c"},
+       {".bz2", "bzip2 -d -c"}};
 
 /*****************************************************************************/
 
-char **infopaths = 0;
-int infopathcount = 0;
+     char **infopaths = 0;
+     int infopathcount = 0;
 
-int
-qsort_cmp (const void *base, const void *compared)
+     int
+       qsort_cmp (const void *base, const void *compared)
 {
   char *cbase = ((TagTable *) base)->nodename;
   char *ccompared = ((TagTable *) compared)->nodename;
@@ -92,7 +92,7 @@ dirpage_lookup (char **type, char ***message, long *lines,
 				  file[fileend - filestart - 1] = 0;
 				  strncpy (name, fileend + 1, dot - fileend - 1);
 				  name[dot - fileend - 1] = 0;
-				  while (*tmp == ' ')
+				  while (isspace (*tmp))
 				    tmp++;
 				  if (strlen (name))
 				    {
@@ -105,7 +105,7 @@ dirpage_lookup (char **type, char ***message, long *lines,
 				    strcat (file, ".info");
 				  id = openinfo (file, 0);
 				  goodHit = 1;
-				  if((nameend - Message[i]) - 2 == filenamelen)	/* the name matches perfectly to the query */
+				  if ((nameend - Message[i]) - 2 == filenamelen)	/* the name matches perfectly to the query */
 				    perfectHit = 1;	/* stop searching for another matches, and use this one */
 				}
 			    }
@@ -272,7 +272,7 @@ load_tag_table (char **message, long lines)
       char *check;
       wsk = message[i + is_indirect];
       check = wsk + strlen (wsk);
-      while (*(++wsk) != ' ')
+      while (!isspace (*(++wsk)))
 	{
 	  if (wsk >= check)
 	    {
