@@ -1,10 +1,21 @@
 #include "common_includes.h"
 
-RCSID("$Id: initializelinks.c,v 1.29 2000/01/27 11:38:15 misiek Exp $")
+RCSID ("$Id: initializelinks.c,v 1.29 2000/01/27 11:38:15 misiek Exp $")
 
 #define MENU_DOT 0
 #define NOTE_DOT 1
 
+     int compare_hyperlink (const void *a, const void *b)
+{
+  return ((HyperObject *) a)->col - ((HyperObject *) b)->col;
+}
+
+void
+sort_hyperlinks_from_current_line (long startlink, long endlink)
+{
+  qsort (hyperobjects + startlink, endlink - startlink, sizeof (HyperObject), compare_hyperlink);
+}
+
 int
 compare_tag_table_string (char *base, char *compared)
 /*
@@ -19,17 +30,20 @@ compare_tag_table_string (char *base, char *compared)
     {
       if (base[i] != compared[j])
 	{
-	  if (((compared[j] != ' ') && (compared[j] != '\t')) ||
-	      ((base[i] != ' ') && (base[i] != '\t')))
+	  if ((isspace (compared[j])) && (isspace (base[i])));	/* OK--two blanks */
+	  else if (isspace (compared[j]))
+	    i--;		/* index of `base' should be unchanged after for's i++ */
+	  else if (isspace (base[i]))
+	    j--;		/* index of `compared' stands in place
+				   and waits for base to skip blanks */
+	  else
 	    return (int) base[i] - (int) compared[j];
-	  i--;
 	}
       j++;
     }
-  while (compared[j])
+  while (compared[j])		/* handle trailing whitespaces of variable `compared' */
     {
-      if (((compared[j] != ' ') && (compared[j] != '\t')) ||
-	  ((base[i] != ' ') && (base[i] != '\t')))
+      if (!isspace (compared[j]))
 	return (int) base[i] - (int) compared[j];
       j++;
     }
@@ -109,12 +123,19 @@ finddot (char *str, int note)	/*
 				 * it may be dot, comma, tab, or newline.
 				 */
 {
+  char *ptr = str;
   char *end[4] =
-  {0, 0, 0, 0};
+  {
+    0, 0, 0, 0
+  };
   char *closest = 0;
   int i;
-  if (strlen (str) < 2)
-    return 0;			/* avoid checking i.e * Menu:\n */
+  while (isspace(*ptr))	/* if there are only spaces and newline... */
+    {
+      if (*ptr == '\n')		/* then it's a `Menu:   \n' entry--skip it */
+	return 0;
+      ptr++;
+    }
   end[0] = strchr (str, '.');	/* nodename entry may end with dot, comma */
   end[1] = strchr (str, ',');	/* tabulation, or newline */
   if (!note)
@@ -180,6 +201,9 @@ initializelinks (char *line1, char *line2, int line)
   char *notestart = 0, *urlstart = 0, *urlend = 0;
   char *quotestart = 0, *quoteend = 0;
   char *buf = xmalloc (strlen (line1) + strlen (line2) + 1);
+  long initialhyperobjectcount = hyperobjectcount;	/* required to sort properly
+							   the hyperlinks from
+							   current line only */
   int changed;
   int line1len = strlen (line1);
   strcpy (buf, line1);		/* copy two lines into one */
@@ -389,7 +413,7 @@ initializelinks (char *line1, char *line2, int line)
 		  hyperobjects = xrealloc (hyperobjects, sizeof (HyperObject) * (hyperobjectcount + 1));
 
 		start = tmp + 1;	/* move after the padding spaces */
-		while (*start == ' ')
+		while (isspace (*start))
 		  start++;
 		NodenameLen = (long) (dot - start);
 		strcpy (hyperobjects[hyperobjectcount].file, "");
@@ -586,7 +610,7 @@ initializelinks (char *line1, char *line2, int line)
 		      hyperobjects = xrealloc (hyperobjects, sizeof (HyperObject) * (hyperobjectcount + 1));
 
 		    start = tmp + 1;	/* move after the padding spaces */
-		    while (*start == ' ')
+		    while (isspace (*start))
 		      start++;
 		    NodenameLen = (long) (dot - start);
 		    strcpy (hyperobjects[hyperobjectcount].file, "");
@@ -644,7 +668,8 @@ initializelinks (char *line1, char *line2, int line)
  * username@something.else[space|\n|\t]                                       *
  *****************************************************************************/
   /* http:// */
-  if ((urlstart = strstr (line1, "http://")) != NULL)
+  urlend = line1;
+  while ((urlstart = strstr (urlend, "http://")) != NULL)
     {
       urlend = findurlend (urlstart);	/* always successful */
       if (!hyperobjectcount)
@@ -662,7 +687,8 @@ initializelinks (char *line1, char *line2, int line)
       hyperobjectcount++;
     }
   /* ftp:// */
-  if ((urlstart = strstr (line1, "ftp://")) != NULL)
+  urlend = line1;
+  while ((urlstart = strstr (urlend, "ftp://")) != NULL)
     {
       urlend = findurlend (urlstart);	/* always successful */
       if (!hyperobjectcount)
@@ -679,6 +705,8 @@ initializelinks (char *line1, char *line2, int line)
       hyperobjects[hyperobjectcount].tagtableoffset = -1;
       hyperobjectcount++;
     }
+  if (initialhyperobjectcount != hyperobjectcount)
+    sort_hyperlinks_from_current_line (initialhyperobjectcount, hyperobjectcount);
   if (buf)
     {
       xfree (buf);
