@@ -1,6 +1,6 @@
 #include "common_includes.h"
 
-RCSID("$Id: mainfunction.c,v 1.38 2000/01/27 11:38:15 misiek Exp $")
+RCSID ("$Id: mainfunction.c,v 1.38 2000/01/27 11:38:15 misiek Exp $")
 
 #include <ctype.h>
 
@@ -8,11 +8,15 @@ RCSID("$Id: mainfunction.c,v 1.38 2000/01/27 11:38:15 misiek Exp $")
 #define        MIN(a,b) (((a)<(b))?(a):(b))
 #endif
 
-void rescan_cursor ();
-int getnodeoffset (int tag_table_pos, int *Indstart);	/* get node offset in file */
+     void rescan_cursor ();	/* set the cursor to 1st item on visible screen */
+     void next_infomenu ();	/* go to the next menu item for sequential reading */
+     int getnodeoffset (int tag_table_pos, int *Indstart);	/* get node offset in file */
 
-int aftersearch = 0;
-long pos, cursor;
+     int aftersearch = 0;
+     int toggled_by_menu = 0;	/* this flag is turned on when the engine
+				   receives a simulated `key.back', caused
+				   by the sequential auto-pgdn reading code */
+     long pos, cursor, infomenu;
 
 WorkRVal
 work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
@@ -28,6 +32,7 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
   int cursorchanged = 0;
   int key = 0;
   int return_value;
+  int statusline = FREE;
   char *token, *tmp;
 
   if (rval.file)		/* if the static variable was allocated, free it */
@@ -41,7 +46,7 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
       rval.node = 0;
     }
 
-  pos = 1, cursor = 0;		/* default position, and selected number */
+  pos = 1, cursor = 0, infomenu = -1;	/* default position, and selected number */
 
 #ifdef getmaxyx
   getmaxyx (stdscr, maxy, maxx);	/* initialize maxx, maxy */
@@ -59,6 +64,8 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
     }
   initializelinks (Message[Lines], "", Lines);
 
+  next_infomenu ();		/* infomenu will remain -1 if it's the last pos, or if there's no menu item */
+
   if (npos != -1)
     pos = npos;			/* set eventual history pos */
 
@@ -68,16 +75,23 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 				 */
     {
       pos = aftersearch;	/* set pos to the found position */
+/*      aftersearch=0; */
     }
 
   if (ncursor != -1)
-    cursor = ncursor;		/* set eventual cursor pos  */
+    {
+      cursor = ncursor;		/* set eventual cursor pos  */
+      infomenu = nmenu;		/* same with last sequential reading menu pos */
+    }
   else
     {
       rescan_cursor ();		/* scan for cursor position */
     }
+  if (toggled_by_menu)		/* this node will not be shown to the user--it shouldn't go to history */
+    dellastinfohistory ();	/* delete the history entry for this node--it's not even seen by the user */
   npos = -1;			/* turn off the `next-time' pos/cursor modifiers */
   ncursor = -1;
+  nmenu = -1;
   addtopline (Type);
   while (1)
     {
@@ -89,13 +103,14 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
       key = pinfo_getch ();
       if (key == ERR)
 	{
-	  showscreen (Message, Type, Lines, pos, cursor);
+	  if (statusline == FREE)
+	    showscreen (Message, Type, Lines, pos, cursor);
 	  nodelay (stdscr, FALSE);
 	  key = pinfo_getch ();
 	}
       else
 	nodelay (stdscr, FALSE);
-
+      statusline = FREE;
 /***************************** keyboard handling ****************************/
       if (key != 0)
 	{
@@ -105,6 +120,46 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 	      if (yesno (_ ("Are you sure to print?"), 0) == 1)
 		printnode (message, lines);
 	    }
+/*==========================================================================*/
+	  if ((key == keys.pgdn_auto_1) ||
+	      (key == keys.pgdn_auto_2) ||
+	      (toggled_by_menu))
+	    {
+	      int wastoggled = toggled_by_menu;
+	      toggled_by_menu = 0;
+	      /* if hyperobject type <= 1, then we have a menu */
+	      if ((pos >= Lines - (maxy - 2)) || (wastoggled))
+		{
+		  if ((infomenu != -1) && (!wastoggled))
+		    {
+		      cursor = infomenu;
+		      key = keys.followlink_1;	/* the handler for keys.followlink must be bellow this statement! */
+		    }
+		  else
+		    /* we shouldn't select a menu item if this node is called via `up:' from bottom, or if there is no menu */
+		    {
+		      char *typestr = strdup (Type);
+		      getnextnode (Type, typestr);
+		      if (strcmp (typestr, ERRNODE) != 0)
+			{
+			  key = keys.nextnode_1;
+			}
+		      else
+			{
+			  getnodename (Type, typestr);
+			  if (strcmp (FirstNodeName, typestr) != 0)	/* if it's not end of all menus */
+			    {
+			      if (wastoggled)	/* if we're in the temporary called up node */
+				toggled_by_menu = KILL_HISTORY;
+			      else	/* if we are calling the up node from non-temporary bottom node */
+				toggled_by_menu = KEEP_HISTORY;
+			      key = keys.upnode_1;
+			      ungetch (KEY_NOTHING);
+			    }
+			}	/* end: else if nextnode==ERRNODE */
+		    }		/* end: if we shouldn't select a menu item */
+		}		/* end: if position is right */
+	    }
 /*==========================================================================*/
 	  if ((key == keys.goline_1) ||
 	      (key == keys.goline_2))
@@ -202,6 +257,7 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 	  if ((key == keys.totalsearch_1) ||	/* search in all nodes later than this one */
 	      (key == keys.totalsearch_2))
 	    {
+	      int tmpaftersearch = aftersearch;
 	      indirectstart = -1;
 	      move (maxy - 1, 0);
 	      attrset (bottomline);
@@ -209,21 +265,15 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 	      curs_set (1);
 	      if (!searchagain.search)	/* if searchagain key wasn't hit */
 		{
-		  token = getstring (_ ("Enter regexp: "));	/* 
-								 * get the
-								 * token 
-								 */
-		  strcpy (searchagain.lastsearch, token);	/* 
-								 * and save 
+		  token = getstring (_ ("Enter regexp: "));	/* get the
+								 * token */
+		  strcpy (searchagain.lastsearch, token);	/* and save 
 								 * it to 
 								 * searchagain
-								 * buffer 
-								 */
-		  searchagain.type = key;	/* 
-						 * give a hint, which key to 
+								 * buffer */
+		  searchagain.type = key;	/* give a hint, which key to 
 						 * ungetch to call this 
-						 * procedure by searchagain 
-						 */
+						 * procedure by searchagain */
 		}
 	      else
 		/* it IS searchagain */
@@ -238,8 +288,8 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 		}
 	      if (strlen (token) == 0)
 		{
-		  token = xrealloc (token, 2);
-		  strcpy (token, " ");
+		  xfree (token);
+		  goto skip_search;
 		}
 	      curs_set (0);
 	      noecho ();
@@ -254,14 +304,18 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 		fileoffset += strlen (Message[i]);
 	      fileoffset += strlen (Type);	/* add also header length */
 
-	      fileoffset += getnodeoffset (tag_table_pos, &indirectstart);
+	      fileoffset += getnodeoffset (tag_table_pos, &indirectstart);	/* also load the variable indirectstart */
 
 	      /*
 	       * Searching part...
 	       */
 
+	      aftersearch = 0;
+
+/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 	      return_value = -1;
-	      if (indirect)
+	      if (indirect)	/* the info is of indirect type;
+				   we'll search through several files */
 		{
 		  FILE *fd;
 		  long tokenpos;
@@ -269,20 +323,16 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 		  long filelen;
 		  for (j = indirectstart; j <= IndirectEntries; j++)
 		    {
-		      fd = openinfo (indirect[j].filename, 1);	/*
-								 * get file
-								 * length.
-								 */
+		      fd = openinfo (indirect[j].filename, 1);	/* get file
+								 * length. */
 		      fseek (fd, 0, SEEK_END);
 		      filelen = ftell (fd);
 
-		      if (j == indirectstart)	/* 
-						 * seek to the beginning of
+		      if (j == indirectstart)	/* seek to the beginning of
 						 * search area. At the first
 						 * time it is `fileoffset',
 						 * then it is the first
-						 * node's offset
-						 */
+						 * node's offset */
 
 			fseek (fd, fileoffset, SEEK_SET);
 		      else
@@ -297,12 +347,10 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 
 		      if (tokenpos != -1)	/* if something was found */
 			{
-			  tokenpos += starttokenpos;	/* 
-							 * add the offset of
+			  tokenpos += starttokenpos;	/* add the offset of
 							 * the part of file,
 							 * which wasn't read
-							 * to the memory
-							 */
+							 * to the memory */
 			  {	/* local scope for tmpvar, matched */
 			    int tmpvar = 0, matched = 0;
 			    tag_table[0].offset = 0;
@@ -345,25 +393,25 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 				    break;
 				}
 			      if (aftersearch > 1)	/* 
-							 * the line where a 
-							 * token is found, is 
-							 * in aftersearch 
+							 * the number ofline where a 
+							 * token is found, is now
+							 * in the variable `aftersearch'
 							 */
 				aftersearch--;
 			      else
 				aftersearch = 1;
 			    }	/* end: if(indirect entry was found) */
-			}
-		      if (aftersearch)	/* if something was found */
-			{
-			  if (tmp)	/* free tmp buffer */
+			  if (aftersearch)	/* if something was found */
 			    {
-			      xfree (tmp);
-			      tmp = 0;
+			      if (tmp)	/* free tmp buffer */
+				{
+				  xfree (tmp);
+				  tmp = 0;
+				}
+			      break;
 			    }
-			  break;
-			}
-		    }		/* end: if(tokenpos) */
+			}	/* end: if(tokenpos) */
+		    }		/* end: indirect file loop */
 		  if (tmp)	/* free tmp buffer */
 		    {
 		      xfree (tmp);
@@ -373,6 +421,7 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 		}		/* end: if(indirect) */
 	      else
 		/* if not indirect */
+/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 		{
 		  long filelen;
 		  long filepos = ftell (id);
@@ -382,15 +431,11 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 		  fseek (id, 0, SEEK_END);	/* calculate filelength */
 		  filelen = ftell (id);
 
-		  fseek (id, fileoffset, SEEK_SET);	/* 
-							 * seek at the start
-							 * of search area.
-							 */
+		  fseek (id, fileoffset, SEEK_SET);	/* seek at the start
+							 * of search area. */
 
-		  starttokenpos = ftell (id);	/* 
-						 * remember the number of
-						 * skipped bytes.
-						 */
+		  starttokenpos = ftell (id);	/* remember the number of
+						 * skipped bytes.*/
 
 		  tmp = xmalloc (filelen - starttokenpos + 10);		/* read data */
 		  fread (tmp, 1, filelen - starttokenpos, id);
@@ -400,11 +445,9 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 
 		  if (tokenpos != -1)	/* if we've found something */
 		    {
-		      tokenpos += starttokenpos;	/* 
-							 * add offset of the start
+		      tokenpos += starttokenpos;	/* add offset of the start
 							 * of search area to this
-							 * token position.
-							 */
+							 * token position. */
 		      {		/* local scope for tmpvar, matched */
 			int tmpvar = 0, matched = 0;
 			tag_table[0].offset = 0;
@@ -420,10 +463,8 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 			  }
 		      }
 		      if (return_value != -1)
-			/* 
-			 * this means, that we've found our entry, and
-			 * we're one position too far with the `i' counter.
-			 */
+			/* this means, that we've found our entry, and
+			 * we're one position too far with the `i' counter. */
 			{
 			  fseek (id, tag_table[return_value].offset, SEEK_SET);
 			  /* seek to the node, which holds found line */
@@ -431,13 +472,11 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 			  fgetc (id);	/* skip newline */
 
 			  aftersearch = 1;
-			  while (ftell (id) < tokenpos)		/* 
-								 * count lines
+			  while (ftell (id) < tokenpos)		/* count lines
 								 * in found
 								 * node, until
 								 * found line
-								 * is met.
-								 */
+								 * is met. */
 			    {
 			      int chr = fgetc (id);
 			      if (chr == '\n')
@@ -449,10 +488,8 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 			    aftersearch--;
 			  else
 			    aftersearch = 1;
-			  fseek (id, filepos, SEEK_SET);	/* 
-								 * seek to old
-								 * filepos.
-								 */
+			  fseek (id, filepos, SEEK_SET);	/* seek to old
+								 * filepos. */
 			}
 		    }		/* end: if(tokenpos) <--> token found */
 		  if (tmp)	/* free tmp buffer */
@@ -461,12 +498,18 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 		      tmp = 0;
 		    }
 		}		/* end: if(!indirect) */
+/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 	      xfree (token);
 	      token = 0;
+
+	      if (!aftersearch)
+		aftersearch = tmpaftersearch;
+
 	      if (return_value != -1)
 		{
 		  infohistory.pos[infohistory.length] = pos;
 		  infohistory.cursor[infohistory.length] = cursor;
+		  infohistory.menu[infohistory.length] = infomenu;
 		  rval.node = xmalloc (strlen (tag_table[return_value].nodename) + 1);
 		  strcpy (rval.node, tag_table[return_value].nodename);
 		  rval.file = xmalloc (1);
@@ -496,8 +539,8 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 		}		/* end of searchagain handler */
 	      if (strlen (token) == 0)
 		{
-		  token = xrealloc (token, 2);
-		  strcpy (token, " ");
+		  xfree (token);
+		  goto skip_search;
 		}
 	      curs_set (0);
 	      noecho ();
@@ -546,6 +589,7 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 	      rescan_cursor ();	/* rescan cursor position in the new place */
 	      aftersearch = 1;
 	    }
+	skip_search:
 /*==========================================================================*/
 	  if ((key == keys.search_again_1) ||	/* search again */
 	      (key == keys.search_again_2))
@@ -588,6 +632,7 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 		  token = 0;
 		  infohistory.pos[infohistory.length] = pos;
 		  infohistory.cursor[infohistory.length] = cursor;
+		  infohistory.menu[infohistory.length] = infomenu;
 		  rval.node = xmalloc (strlen (tag_table[return_value].nodename) + 1);
 		  strcpy (rval.node, tag_table[return_value].nodename);
 		  rval.file = xmalloc (1);
@@ -656,7 +701,7 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 		      move (0, 0);
 		    }
 		}
-	      getch ();
+	      statusline = LOCKED;
 	      xfree (token);
 	      token = 0;
 	    }
@@ -673,6 +718,7 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 		{
 		  infohistory.pos[infohistory.length] = pos;
 		  infohistory.cursor[infohistory.length] = cursor;
+		  infohistory.menu[infohistory.length] = infomenu;
 		  rval.node = xmalloc (strlen (tag_table[return_value].nodename) + 1);
 		  strcpy (rval.node, tag_table[return_value].nodename);
 		  rval.file = xmalloc (1);
@@ -694,6 +740,7 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 		{
 		  infohistory.pos[infohistory.length] = pos;
 		  infohistory.cursor[infohistory.length] = cursor;
+		  infohistory.menu[infohistory.length] = infomenu;
 		  rval.node = xmalloc (strlen (tag_table[return_value].nodename) + 1);
 		  strcpy (rval.node, tag_table[return_value].nodename);
 		  rval.file = xmalloc (1);
@@ -713,8 +760,12 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 	      token = 0;
 	      if (return_value != -1)
 		{
-		  infohistory.pos[infohistory.length] = pos;
-		  infohistory.cursor[infohistory.length] = cursor;
+		  if (toggled_by_menu == KEEP_HISTORY)
+		    {
+		      infohistory.pos[infohistory.length] = pos;
+		      infohistory.cursor[infohistory.length] = cursor;
+		      infohistory.menu[infohistory.length] = infomenu;
+		    }
 		  rval.node = xmalloc (strlen (tag_table[return_value].nodename) + 1);
 		  strcpy (rval.node, tag_table[return_value].nodename);
 		  rval.file = xmalloc (1);
@@ -804,13 +855,6 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 		  cursor = hyperobjectcount - 1;
 		}
 	    }
-/*==========================================================================*/
-	  if ((key == keys.pgdn_auto_1) ||
-	      (key == keys.pgdn_auto_2))
-	    {
-	      if (pos >= Lines - (maxy - 1))
-		ungetch (keys.nextnode_1);
-	    }
 /*==========================================================================*/
 	  if ((key == keys.home_1) ||
 	      (key == keys.home_2))
@@ -822,8 +866,8 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 	  if ((key == keys.pgup_1) |
 	      (key == keys.pgup_2))
 	    {
-	      if (pos > (maxy - 1))
-		pos -= (maxy - 1);
+	      if (pos > (maxy - 2))
+		pos -= (maxy - 2);
 	      else
 		pos = 1;
 	      rescan_cursor ();
@@ -863,7 +907,7 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 		  }
 	      if (!cursorchanged)
 		{
-		  if (pos <= Lines - (maxy - 1))
+		  if (pos <= Lines - (maxy - 2))
 		    pos++;
 		  for (i = cursor + 1; i < hyperobjectcount; i++)
 		    {
@@ -886,6 +930,7 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 	    {
 	      infohistory.pos[infohistory.length] = pos;
 	      infohistory.cursor[infohistory.length] = cursor;
+	      infohistory.menu[infohistory.length] = infomenu;
 	      rval.node = xmalloc (strlen (FirstNodeName) + 1);
 	      strcpy (rval.node, FirstNodeName);
 	      rval.file = xmalloc (1);
@@ -910,6 +955,7 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 
 		  npos = infohistory.pos[infohistory.length];
 		  ncursor = infohistory.cursor[infohistory.length];
+		  nmenu = infohistory.menu[infohistory.length];
 		  dellastinfohistory ();	/* remove history entry for previous node */
 		  aftersearch = 0;
 		  return rval;
@@ -921,10 +967,15 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 	    {
 	      infohistory.pos[infohistory.length] = pos;
 	      infohistory.cursor[infohistory.length] = cursor;
+	      infohistory.menu[infohistory.length] = infomenu;
+	      if (!toggled_by_menu)
+		infohistory.menu[infohistory.length] = cursor;
 	      if ((cursor >= 0) && (cursor < hyperobjectcount))
 		if ((hyperobjects[cursor].line >= pos) &&
-		    (hyperobjects[cursor].line < pos + (maxy - 1)))
+		    (hyperobjects[cursor].line < pos + (maxy - 2)) ||
+		    (toggled_by_menu))
 		  {
+		    toggled_by_menu = 0;
 		    if (hyperobjects[cursor].type < 4)	/* normal info link */
 		      {
 			rval.node = xmalloc (strlen (hyperobjects[cursor].node) + 1);
@@ -1096,6 +1147,26 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
   aftersearch = 0;
   return rval;
 }
+
+void
+next_infomenu ()
+{
+  int i;
+  if (hyperobjectcount == 0)
+    {
+      infomenu = -1;
+      return;
+    }
+  for (i = infomenu + 1; i < hyperobjectcount; i++)
+    {
+      if (hyperobjects[i].type <= 1)	/* menu item */
+	{
+	  infomenu = i;
+	  return;
+	}
+    }
+  infomenu = -1;		/* no menuitem left is found */
+}
 void
 rescan_cursor ()
 {
@@ -1103,7 +1174,7 @@ rescan_cursor ()
   for (i = 0; i < hyperobjectcount; i++)
     {
       if ((hyperobjects[i].line >= pos) &&
-	  (hyperobjects[i].line < pos + (maxy - 1)))
+	  (hyperobjects[i].line < pos + (maxy - 2)))
 	{
 	  if (hyperobjects[i].type < HIGHLIGHT)
 	    {
