@@ -1,6 +1,6 @@
 #include "common_includes.h"
 
-RCSID("$Id: manual.c,v 1.59 2000/01/27 11:38:15 misiek Exp $")
+RCSID ("$Id: manual.c,v 1.59 2000/01/27 11:38:15 misiek Exp $")
 
 #include <ctype.h>
 #include <sys/stat.h>
@@ -9,15 +9,16 @@ RCSID("$Id: manual.c,v 1.59 2000/01/27 11:38:15 misiek Exp $")
 #define FTPSECTION 101
 #define MAILSECTION 102
 
-void loadmanual (FILE * id);	/* load manual */
-int manualwork ();		/* handle keyboard */
-void rescan_selected ();	/* scan for potential link to select on
+     int ishyphen (unsigned char ch);	/* check if a char is a hyphen character */
+     void loadmanual (FILE * id);	/* load manual */
+     int manualwork ();		/* handle keyboard */
+     void rescan_selected ();	/* scan for potential link to select on
 				   viewed manual page */
-void showmanualscreen ();	/* self explanatory */
-void mvaddstr_manual (int y, int x, char *str);		/* mvaddstr with bold/italic */
-void add_highlights ();		/* adds highlights to a painted screen */
-void strip_manual (char *buf);	/* strips line from formatting characters */
-void man_initializelinks (char *line, int carry);
+     void showmanualscreen ();	/* self explanatory */
+     void mvaddstr_manual (int y, int x, char *str);	/* mvaddstr with bold/italic */
+     void add_highlights ();	/* adds highlights to a painted screen */
+     void strip_manual (char *buf);	/* strips line from formatting characters */
+     void man_initializelinks (char *line, int carry);
  /* 
   * Initialize links in a line .
   * Links are entries of form
@@ -25,62 +26,62 @@ void man_initializelinks (char *line, int carry);
   * in `manuallinks' var, described 
   * bellow.
   */
-int is_in_manlinks (char *in, char *find);
+     int is_in_manlinks (char *in, char *find);
 
-void printmanual (char **Message, long Lines);
+     void printmanual (char **Message, long Lines);
 
-char **manual = 0;		/* line by line stored manual */
-int ManualLines = 0;		/* number of lines in manual */
-int selected = -1;		/* number of selected link (offset in 'manuallinks',
+     char **manual = 0;		/* line by line stored manual */
+     int ManualLines = 0;	/* number of lines in manual */
+     int selected = -1;		/* number of selected link (offset in 'manuallinks',
 				   bellow) */
-int manualpos = 0;		/* number of the first line, which is painted on
+     int manualpos = 0;		/* number of the first line, which is painted on
 				   screen */
 
-int manual_aftersearch = 0;	/* this is set if man page is now after search 
-				   operation */
-int manwidthChanged = 0;	/* this flag indicates whether the env variable
+     int manual_aftersearch = 0;	/* this is set if man page is now after search 
+					   operation */
+     int manwidthChanged = 0;	/* this flag indicates whether the env variable
 				   $MANWIDTH was changed by pinfo */
 
-typedef struct
-  {
-    char name[256];		/* name of a manual */
-    char sect[32];		/* section */
-    int selected;		/* what was last selected on this page */
-    int pos;			/* what was the last manualpos */
-  }
+     typedef struct
+       {
+	 char name[256];	/* name of a manual */
+	 char sect[32];		/* section */
+	 int selected;		/* what was last selected on this page */
+	 int pos;		/* what was the last manualpos */
+       }
 manhistory;			/* 
 				 * type for the `lastread' history entries, when viewing
 				 * man pages.
 				 */
 
-manhistory *manualhistory = 0;	/* manual lastread history */
-int manualhistorylength = 0;	/* length of the above table - 1 */
+     manhistory *manualhistory = 0;	/* manual lastread history */
+     int manualhistorylength = 0;	/* length of the above table - 1 */
 
-typedef struct
-  {				/* struct for hypertext references */
-    int line;			/* line of the manpage, where the reference
+     typedef struct
+       {			/* struct for hypertext references */
+	 int line;		/* line of the manpage, where the reference
 				   is */
-    int col;			/* column of that line */
-    char *name;			/* name of the reference */
-    char section[32];		/* section of the reference */
-    int section_mark;
-    int carry;			/* determine whether there is a hyphen above */
-  }
+	 int col;		/* column of that line */
+	 char *name;		/* name of the reference */
+	 char section[32];	/* section of the reference */
+	 int section_mark;
+	 int carry;		/* determine whether there is a hyphen above */
+       }
 manuallink;			/*
 				 * this structure describes a hyperlink
 				 * in manual viewer
 				 */
 
-manuallink *manuallinks = 0;	/* a set of manual references of man page */
+     manuallink *manuallinks = 0;	/* a set of manual references of man page */
 
-int ManualLinks = 0;		/* number of found manual references in man page */
+     int ManualLinks = 0;	/* number of found manual references in man page */
 
-int historical = 0;		/* semaphore for checking if it's a history (left
+     int historical = 0;	/* semaphore for checking if it's a history (left
 				   arrow) call */
 
 
-void
-manual_free_buffers ()		/* free buffers allocated by current man page */
+     void
+       manual_free_buffers ()	/* free buffers allocated by current man page */
 {
   int i;
   if (manual)			/* first free previously allocated memory */
@@ -131,20 +132,42 @@ construct_manualname (char *buf, int which)	/* construct man name; take
     }
   else
     {
-      char *base = xmalloc (1024);
-      char *ptr;
-      int tmppos;
-      strcpy (base, manual[manuallinks[which].line - 1]);
-      strip_manual (base);
-      ptr = base + strlen (base) - 3;
-      while (((isalpha (*ptr)) || (*ptr == '.')) && (ptr > base))
-	ptr--;
-      strcpy (buf, ptr);
-      tmppos = strlen (buf);
-      if (tmppos > 1);
-      buf[tmppos - 2] = 0;
-      strcat (buf, manuallinks[which].name);
-      xfree (base);
+      if (manuallinks[which].section_mark < HTTPSECTION)	/* normal manual reference */
+	{
+	  char *base = xmalloc (1024);
+	  char *ptr;
+	  int tmppos;
+	  strcpy (base, manual[manuallinks[which].line - 1]);
+	  strip_manual (base);
+	  ptr = base + strlen (base) - 3;
+	  while (((isalpha (*ptr)) || (*ptr == '.')) && (ptr > base))
+	    ptr--;
+	  strcpy (buf, ptr);
+	  tmppos = strlen (buf);
+	  if (tmppos > 1);
+	  buf[tmppos - 2] = 0;
+	  strcat (buf, manuallinks[which].name);
+	  xfree (base);
+	}
+      else
+	/* url reference */
+	{
+	  char *base = xmalloc (1024);
+	  char *ptr, *eptr;
+	  int tmppos;
+	  int namelen = strlen (manuallinks[which].name);
+	  strcpy (base, manual[manuallinks[which].line + 1]);
+	  strip_manual (base);
+	  ptr = base;
+	  while (isspace (*ptr))
+	    ptr++;		/* skip whitespace */
+	  eptr = findurlend (ptr);
+	  *eptr = 0;
+	  strcpy (buf, manuallinks[which].name);
+	  buf[namelen - 1] = 0;	/* cut the hyphen */
+	  strcat (buf, ptr);
+	  xfree (base);
+	}
     }
 }
 
@@ -256,7 +279,8 @@ handlemanual (char *name)	/*
 	    )
 	  )
 	  {
-	    printf ("%s %s\n", _ ("Calling gunzip for"), location);
+	    if (verbose)
+	      printf ("%s %s\n", _ ("Calling gunzip for"), location);
 	    snprintf (cmd, 255, "gunzip -c %s", location);
 	    source = popen (cmd, "r");
 	    zipped = 1;
@@ -526,7 +550,7 @@ loadmanual (FILE * id)		/* loads manual from given filedescriptor */
 	  man_initializelinks (tmp, carryflag);
 	  carryflag = 0;
 	  if (manlinelen > 1)
-	    if (manual[ManualLines][manlinelen - 2] == '-')
+	    if (ishyphen (manual[ManualLines][manlinelen - 2]))
 	      carryflag = 1;
 	  xfree (tmp);		/* free temporary buffer */
 	  prevlinechar = manual[ManualLines][0];
@@ -540,17 +564,32 @@ loadmanual (FILE * id)		/* loads manual from given filedescriptor */
 
 }
 
+int
+compare_manuallink (const void *a, const void *b)
+{
+  return ((manuallink *) a)->col - ((manuallink *) b)->col;
+}
+
+void
+sort_manuallinks_from_current_line (long startlink, long endlink)
+{
+  qsort (manuallinks + startlink, endlink - startlink, sizeof (manuallink), compare_manuallink);
+}
+
+
 void
 man_initializelinks (char *tmp, int carry)	/* initializes hyperlinks in manual */
 {
   int tmpcnt = strlen (tmp) + 1;	/* set tmpcnt to the trailing zero of tmp */
   char *link = tmp;
   char *urlstart, *urlend;
+  long initialManualLinks = ManualLinks;
   int i, b;
 /******************************************************************************
  * handle url refrences                                                       *
  *****************************************************************************/
-  if ((urlstart = strstr (tmp, "http://")) != NULL)
+  urlend = tmp;
+  while ((urlstart = strstr (urlend, "http://")) != NULL)
     {
       urlend = findurlend (urlstart);	/* always successfull */
       manuallinks = xrealloc (manuallinks, sizeof (manuallink) * (ManualLinks + 3));
@@ -561,9 +600,14 @@ man_initializelinks (char *tmp, int carry)	/* initializes hyperlinks in manual *
       manuallinks[ManualLinks].name = xmalloc (urlend - urlstart + 10);
       strncpy (manuallinks[ManualLinks].name, urlstart, urlend - urlstart);
       manuallinks[ManualLinks].name[urlend - urlstart] = 0;
+      if (ishyphen (manuallinks[ManualLinks].name[urlend - urlstart - 1]))
+	manuallinks[ManualLinks].carry = 1;
+      else
+	manuallinks[ManualLinks].carry = 0;
       ManualLinks++;
     }
-  if ((urlstart = strstr (tmp, "ftp://")) != NULL)
+  urlend = tmp;
+  while ((urlstart = strstr (urlend, "ftp://")) != NULL)
     {
       urlend = findurlend (urlstart);	/* always successfull */
       manuallinks = xrealloc (manuallinks, sizeof (manuallink) * (ManualLinks + 3));
@@ -574,9 +618,14 @@ man_initializelinks (char *tmp, int carry)	/* initializes hyperlinks in manual *
       manuallinks[ManualLinks].name = xmalloc (urlend - urlstart + 10);
       strncpy (manuallinks[ManualLinks].name, urlstart, urlend - urlstart);
       manuallinks[ManualLinks].name[urlend - urlstart] = 0;
+      if (ishyphen (manuallinks[ManualLinks].name[urlend - urlstart - 1]))
+	manuallinks[ManualLinks].carry = 1;
+      else
+	manuallinks[ManualLinks].carry = 0;
       ManualLinks++;
     }
-  if ((urlstart = findemailstart (tmp)) != NULL)
+  urlend = tmp;
+  while ((urlstart = findemailstart (urlend)) != NULL)
     {
       urlend = findurlend (urlstart);	/* always successfull */
       manuallinks = xrealloc (manuallinks, sizeof (manuallink) * (ManualLinks + 3));
@@ -587,7 +636,12 @@ man_initializelinks (char *tmp, int carry)	/* initializes hyperlinks in manual *
       manuallinks[ManualLinks].name = xmalloc (urlend - urlstart + 10);
       strncpy (manuallinks[ManualLinks].name, urlstart, urlend - urlstart);
       manuallinks[ManualLinks].name[urlend - urlstart] = 0;
-      if (strchr (manuallinks[ManualLinks].name, '.') != NULL)
+      if (ishyphen (manuallinks[ManualLinks].name[urlend - urlstart - 1]))
+	manuallinks[ManualLinks].carry = 1;
+      else
+	manuallinks[ManualLinks].carry = 0;
+      if (strchr (manuallinks[ManualLinks].name, '.') != NULL)	/* there should be a 
+								   dot in e-mail domain */
 	ManualLinks++;
     }
 /******************************************************************************
@@ -616,7 +670,7 @@ man_initializelinks (char *tmp, int carry)	/* initializes hyperlinks in manual *
 		    i++;
 		  for (; i > 0; --i)
 		    {
-		      if (tmp[i] != ' ')
+		      if (!isspace (tmp[i]))
 			/* ignore spaces between linkname and '(x)' */
 			break;
 		    }
@@ -625,7 +679,7 @@ man_initializelinks (char *tmp, int carry)	/* initializes hyperlinks in manual *
 		  tmp[breakpos] = 0;
 		  for (i = breakpos; i > 0; --i)	/* scan to the first space sign or to 0 -- that means go to the beginning of the scanned token */
 		    {
-		      if (tmp[i] == ' ')
+		      if (isspace (tmp[i]))
 			{
 			  i++;
 			  break;
@@ -656,7 +710,7 @@ man_initializelinks (char *tmp, int carry)	/* initializes hyperlinks in manual *
 						 * the first word of this line */
 		    {
 		      if (b > 0)
-			if ((tmp[b] != ' ') && (tmp[b] != '\t'))
+			if (!isspace (tmp[b] != ' '))
 			  break;
 		    }
 		  if (b >= 0)
@@ -677,6 +731,8 @@ man_initializelinks (char *tmp, int carry)	/* initializes hyperlinks in manual *
     }
   while (link != NULL);		/* do this line until strchr() won't
 				   find a '(' in string */
+  if (initialManualLinks != ManualLinks)
+    sort_manuallinks_from_current_line (initialManualLinks, ManualLinks);
 }
 
 int
@@ -884,8 +940,8 @@ manualwork ()			/* viewer function. Handles keyboard actions--main event loop */
 		}		/* end of searchagain handler */
 	      if (strlen (token) == 0)
 		{
-		  token = xrealloc (token, 2);
-		  strcpy (token, " ");
+		  xfree (token);
+		  goto skip_search;
 		}
 	      curs_set (0);
 	      noecho ();
@@ -942,6 +998,7 @@ manualwork ()			/* viewer function. Handles keyboard actions--main event loop */
 		  ungetch (searchagain.type);
 		}
 	    }
+	skip_search:
 /*===========================================================================*/
 	  if ((key == keys.twoup_1) ||
 	      (key == keys.twoup_2))
@@ -994,8 +1051,8 @@ manualwork ()			/* viewer function. Handles keyboard actions--main event loop */
 	      (key == keys.end_2))
 	    {
 	      manualpos = ManualLines - (maxy - 1);
-	      if (manualpos < 0) 
-	        manualpos = 0;
+	      if (manualpos < 0)
+		manualpos = 0;
 	      selected = ManualLinks - 1;
 	    }
 /*===========================================================================*/
@@ -1094,10 +1151,12 @@ manualwork ()			/* viewer function. Handles keyboard actions--main event loop */
 		  {
 		    if (!strncmp (manuallinks[selected].section, "HTTPSECTION", 11))
 		      {
-			char *tempbuf = xmalloc (strlen (manuallinks[selected].name) + strlen (httpviewer) + 10);
+			int buflen;
+			char *tempbuf = xmalloc (1024);
 			strcpy (tempbuf, httpviewer);
 			strcat (tempbuf, " ");
-			strcat (tempbuf, manuallinks[selected].name);
+			buflen = strlen (tempbuf);
+			construct_manualname (tempbuf + buflen, selected);
 			myendwin ();
 			system (tempbuf);
 			doupdate ();
@@ -1105,10 +1164,12 @@ manualwork ()			/* viewer function. Handles keyboard actions--main event loop */
 		      }
 		    else if (!strncmp (manuallinks[selected].section, "FTPSECTION", 10))
 		      {
-			char *tempbuf = xmalloc (strlen (manuallinks[selected].name) + strlen (ftpviewer) + 10);
-			strcpy (tempbuf, httpviewer);
+			int buflen;
+			char *tempbuf = xmalloc (1024);
+			strcpy (tempbuf, ftpviewer);
 			strcat (tempbuf, " ");
-			strcat (tempbuf, manuallinks[selected].name);
+			buflen = strlen (tempbuf);
+			construct_manualname (tempbuf + buflen, selected);
 			myendwin ();
 			system (tempbuf);
 			doupdate ();
@@ -1116,12 +1177,13 @@ manualwork ()			/* viewer function. Handles keyboard actions--main event loop */
 		      }
 		    else if (!strncmp (manuallinks[selected].section, "MAILSECTION", 11))
 		      {
-			char *tempbuf = xmalloc (strlen (manuallinks[selected].name) + strlen (maileditor) + 10);
+			int buflen;
+			char *tempbuf = xmalloc (1024);
 			strcpy (tempbuf, maileditor);
 			strcat (tempbuf, " ");
-			strcat (tempbuf, manuallinks[selected].name);
+			buflen = strlen (tempbuf);
+			construct_manualname (tempbuf + buflen, selected);
 			myendwin ();
-			system ("clear");
 			system (tempbuf);
 			doupdate ();
 			xfree (tempbuf);
@@ -1266,6 +1328,7 @@ showmanualscreen ()		/* show the currently visible part of manpage */
 #ifdef getmaxyx
   getmaxyx (stdscr, maxy, maxx);	/* refresh maxy, maxx values */
 #endif
+  attrset (normal);
   for (i = manualpos; (i < manualpos + (maxy - 2)) && (i < ManualLines); i++)
     /* print all visible text lines */
     {
@@ -1277,7 +1340,7 @@ showmanualscreen ()		/* show the currently visible part of manpage */
 	manual[i][len - 1] = '\n';
     }
 #ifdef HAVE_BKGDSET
-  bkgdset (' '|normal);
+  bkgdset (' ' | normal);
 #endif
   clrtobot ();			/* and clear to bottom */
 #ifdef HAVE_BKGDSET
@@ -1313,8 +1376,8 @@ mvaddstr_manual (int y, int x, char *str)	/* print a manual line */
 	  if ((str[i] == 8) && (str[i - 1] == '_'))	/* handle bold highlight */
 	    {
 	      attrset (manualbold);
-	      addch (str[i] & A_CHARTEXT);
-	      addch (str[i + 1] & A_CHARTEXT);
+	      addch (str[i] & 0xff);
+	      addch (str[i + 1] & 0xff);
 	      attrset (normal);
 	      i++;
 	      goto label_skip_other;
@@ -1328,7 +1391,7 @@ mvaddstr_manual (int y, int x, char *str)	/* print a manual line */
 	  else
 	    /* unhighlighted */
 	    {
-	      addch (str[i] & A_CHARTEXT);
+	      addch (str[i] & 0xff);
 	      goto label_skip_other;
 	    }
 	}
@@ -1338,13 +1401,13 @@ mvaddstr_manual (int y, int x, char *str)	/* print a manual line */
 	  if ((str[i + 1] == 8) && (str[i + 2] == str[i]))
 	    {
 	      attrset (manualitalic);
-	      addch (str[i] & A_CHARTEXT);
+	      addch (str[i] & 0xff);
 	      i += 2;
 	      attrset (normal);
 	    }
 	  else
 	    {
-	      addch (str[i] & A_CHARTEXT);
+	      addch (str[i] & 0xff);
 	    }
 	}
     label_skip_other:;
@@ -1370,9 +1433,9 @@ mvaddstr_manual (int y, int x, char *str)	/* print a manual line */
 	    {
 	      int n = pmatch[0].rm_eo - pmatch[0].rm_so, k;
 	      int rx = pmatch[0].rm_so + tmpstr - strippedline;
-	      int curY,curX;
+	      int curY, curX;
 	      char tmpchr;
-	      getyx(stdscr,curY,curX);
+	      getyx (stdscr, curY, curX);
 	      tmpchr = strippedline[rx + n];
 	      strippedline[rx + n] = 0;
 	      attrset (searchhighlight);
@@ -1380,7 +1443,7 @@ mvaddstr_manual (int y, int x, char *str)	/* print a manual line */
 	      attrset (normal);
 	      strippedline[rx + n] = tmpchr;
 	      tmpstr = tmpstr + pmatch[0].rm_eo;
-	      move(curY,curX);
+	      move (curY, curX);
 	    }
 	}
     }
@@ -1401,6 +1464,33 @@ add_highlights ()		/* add hyperobject highlights */
 		attrset (noteselected);
 	      else
 		attrset (note);
+	      if (manuallinks[i].carry == 1)
+		{
+		  int x, y, ltline = manuallinks[i].line - 1;
+		  char *tmpstr = strdup (manual[ltline]);
+		  int ltlinelen;
+		  char *newlinemark;
+		  strip_manual (tmpstr);
+		  ltlinelen = strlen (tmpstr);
+		  newlinemark = tmpstr + ltlinelen - 1;
+		  getyx (stdscr, y, x);
+		  if (y > 2)
+		    {
+#define TestCh manual[ltline][ltlinelen]
+		      if (ltlinelen > 2)
+			ltlinelen -= 3;
+		      while ((isalpha (TestCh)) || (TestCh == '.'))
+			ltlinelen--;
+
+		      *newlinemark = 0;
+		      mvaddstr (manuallinks[i].line - manualpos + 1 - 1, ltlinelen,
+				&tmpstr[ltlinelen]);
+		      *newlinemark = '\n';
+#undef TestCh
+		    }
+		  xfree (tmpstr);
+		  move (y, x);
+		}
 	    }
 	  else
 	    {
@@ -1408,33 +1498,18 @@ add_highlights ()		/* add hyperobject highlights */
 		attrset (urlselected);
 	      else
 		attrset (url);
-	    }
-	  if (manuallinks[i].carry == 1)
-	    {
-	      int x, y, ltline = manuallinks[i].line - 1;
-	      char *tmpstr = strdup (manual[ltline]);
-	      int ltlinelen;
-	      char *newlinemark;
-	      strip_manual (tmpstr);
-	      ltlinelen = strlen (tmpstr);
-	      newlinemark = tmpstr + ltlinelen - 1;
-	      getyx (stdscr, y, x);
-	      if (y > 2)
+	      if (manuallinks[i].carry == 1)
 		{
-#define TestCh manual[ltline][ltlinelen]
-		  if (ltlinelen > 2)
-		    ltlinelen -= 3;
-		  while ((isalpha (TestCh)) || (TestCh == '.'))
-		    ltlinelen--;
-
-		  *newlinemark = 0;
-		  mvaddstr (manuallinks[i].line - manualpos + 1 - 1, ltlinelen,
-			    &tmpstr[ltlinelen]);
-		  *newlinemark = '\n';
-#undef TestCh
+		  int x, y, ltline = manuallinks[i].line + 1;
+		  char *tmpstr = strdup (manual[ltline]);
+		  char *wsk = tmpstr, *wskend;
+		  strip_manual (tmpstr);
+		  while (isspace (*wsk))
+		    wsk++;	/* skip spaces */
+		  wskend = findurlend (wsk);	/* find the end of url */
+		  *wskend = 0;	/* add end of string, and print */
+		  mvaddstr (manuallinks[i].line - manualpos + 2, wsk - tmpstr, wsk);
 		}
-	      xfree (tmpstr);
-	      move (y, x);
 	    }
 	  mvaddstr (1 + manuallinks[i].line - manualpos, manuallinks[i].col, manuallinks[i].name);
 	  attrset (normal);
@@ -1476,7 +1551,11 @@ is_in_manlinks (char *in, char *find)	/* checks if a construction, which
     {
       while ((token = strtok (NULL, delimiters)))
 	{
+#ifdef HAVE_STRCASECMP
+	  if (!strcasecmp (token, find))
+#else
 	  if (!strcmp (token, find))
+#endif	  
 	    {
 	      xfree ((void *) copy);
 	      return 0;
@@ -1506,3 +1585,11 @@ printmanual (char **Message, long Lines)
     }
   pclose (prnFD);
 }
+
+int
+ishyphen (unsigned char ch)
+{
+  if ((ch == '-') || (ch == SOFT_HYPHEN))
+    return 1;
+  return 0;
+}
