@@ -1,79 +1,80 @@
+
 #include "common_includes.h"
 
-RCSID("$Id: parse_config.c,v 1.41 2000/01/27 11:38:15 misiek Exp $")
+RCSID ("$Id: parse_config.c,v 1.41 2000/01/27 11:38:15 misiek Exp $")
 
 #include <ctype.h>
 
 #define COLOR_DEFAULT -1	/* mutt uses this was for transparency */
 
-regex_t *h_regexp = 0;		/* regexps to highlight */
-int h_regexp_num = 0;		/* number of those regexps */
+     regex_t *h_regexp = 0;	/* regexps to highlight */
+     int h_regexp_num = 0;	/* number of those regexps */
 
-extern int use_apropos;
-extern int CutManHeaders;
-extern int DontHandleWithoutTagTable;
-extern int use_manual;
-extern int use_raw_filename;
-extern int quote_ignored;
-extern char *httpviewer;
-extern char *ftpviewer;
-extern char *maileditor;
-extern char *manlinks;
-extern char *ignoredmacros;
+     extern int use_apropos;
+     extern int CutManHeaders;
+     extern int DontHandleWithoutTagTable;
+     extern int use_manual;
+     extern int use_raw_filename;
+     extern int quote_ignored;
+     extern char *httpviewer;
+     extern char *ftpviewer;
+     extern char *maileditor;
+     extern char *manlinks;
+     extern char *ignoredmacros;
 
-struct keybindings keys =
-{'s', 'S',			/* regexp search */
- '/', '/',			/* regexp search, this page */
- 'g', 'G',			/* goto node */
- 'p', 'P',			/* previous node */
- 'n', 'N',			/* next node */
- 'u', 'U',			/* up node */
- KEY_UP, 'k',			/* up one line */
+     struct keybindings keys =
+     {'s', 'S',			/* regexp search */
+      '/', '/',			/* regexp search, this page */
+      'g', 'G',			/* goto node */
+      'p', 'P',			/* previous node */
+      'n', 'N',			/* next node */
+      'u', 'U',			/* up node */
+      KEY_UP, 'k',		/* up one line */
 #ifdef HAVE_KEY_END
- KEY_END
+      KEY_END
 #else				/* HAVE_KEY_END */
- 'E'
+      'E'
 #endif				/* ~HAVE_KEY_END */
- ,'e',				/* end */
- KEY_NPAGE, ' ',		/* down one page */
- KEY_HOME, 'H',			/* home */
- KEY_PPAGE, '-',		/* up one page */
- KEY_DOWN, 'j',			/* down one line */
- 't', 'T',			/* top */
- KEY_LEFT, 'h',			/* back */
- KEY_RIGHT, '\n',		/* follow link */
- 'Q', 'q',			/* quit */
- 12, '~',			/* refresh screen -- 12 is C-L */
- '!', '!',			/* shell feed */
- 'D', 'd',			/* goto dir page */
- 0, ' ',			/* pgdn_auto */
- 0, '-',			/* pgup_auto */
- 'f', 0,			/* search again */
- 'l', 0,			/* go to line */
- KEY_IC, 0,			/* two lines up */
- KEY_DC, 1,			/* two lines down */
- ']', 0				/* print */
-};
+      ,'e',			/* end */
+      KEY_NPAGE, ' ',		/* down one page */
+      KEY_HOME, 'H',		/* home */
+      KEY_PPAGE, '-',		/* up one page */
+      KEY_DOWN, 'j',		/* down one line */
+      't', 'T',			/* top */
+      KEY_LEFT, 'h',		/* back */
+      KEY_RIGHT, '\n',		/* follow link */
+      'Q', 'q',			/* quit */
+      12, '~',			/* refresh screen -- 12 is C-L */
+      '!', '!',			/* shell feed */
+      'D', 'd',			/* goto dir page */
+      0, ' ',			/* pgdn_auto */
+      0, '-',			/* pgup_auto */
+      'f', 0,			/* search again */
+      'l', 0,			/* go to line */
+      KEY_IC, 0,		/* two lines up */
+      KEY_DC, 1,		/* two lines down */
+      ']', 0			/* print */
+     };
 
 #ifndef NO_COLOR_CURSES
-struct colours cols =
-{COLOR_WHITE, COLOR_BLACK, NO_BOLD, NO_BLINK,	/* normal */
- COLOR_GREEN, COLOR_WHITE, BOLD, NO_BLINK,	/* selected menu */
- COLOR_GREEN, COLOR_BLACK, BOLD, NO_BLINK,	/* menu */
- COLOR_BLUE, COLOR_WHITE, BOLD, NO_BLINK,	/* selected note */
- COLOR_BLUE, COLOR_BLACK, BOLD, NO_BLINK,	/* note */
- COLOR_GREEN, COLOR_BLUE, BOLD, NO_BLINK,	/* top line */
- COLOR_GREEN, COLOR_BLUE, BOLD, NO_BLINK,	/* bottom line */
- COLOR_YELLOW, COLOR_BLACK, BOLD, NO_BLINK,	/* manual bold */
- COLOR_WHITE, COLOR_BLACK, BOLD, NO_BLINK,	/* manual italic */
- COLOR_MAGENTA, COLOR_BLACK, BOLD, NO_BLINK,	/* url */
- COLOR_MAGENTA, COLOR_GREEN, NO_BOLD, NO_BLINK,		/* url selected */
- COLOR_WHITE, COLOR_BLACK, BOLD, NO_BLINK,	/* info highlight (quoted text) */
- COLOR_YELLOW, COLOR_BLACK, BOLD, NO_BLINK	/* search highlight */
-};
+     struct colours cols =
+     {COLOR_WHITE, COLOR_BLACK, NO_BOLD, NO_BLINK,	/* normal */
+      COLOR_GREEN, COLOR_WHITE, BOLD, NO_BLINK,		/* selected menu */
+      COLOR_GREEN, COLOR_BLACK, BOLD, NO_BLINK,		/* menu */
+      COLOR_BLUE, COLOR_WHITE, BOLD, NO_BLINK,	/* selected note */
+      COLOR_BLUE, COLOR_BLACK, BOLD, NO_BLINK,	/* note */
+      COLOR_GREEN, COLOR_BLUE, BOLD, NO_BLINK,	/* top line */
+      COLOR_GREEN, COLOR_BLUE, BOLD, NO_BLINK,	/* bottom line */
+      COLOR_YELLOW, COLOR_BLACK, BOLD, NO_BLINK,	/* manual bold */
+      COLOR_WHITE, COLOR_BLACK, BOLD, NO_BLINK,		/* manual italic */
+      COLOR_MAGENTA, COLOR_BLACK, BOLD, NO_BLINK,	/* url */
+      COLOR_MAGENTA, COLOR_GREEN, NO_BOLD, NO_BLINK,	/* url selected */
+      COLOR_WHITE, COLOR_BLACK, BOLD, NO_BLINK,		/* info highlight (quoted text) */
+      COLOR_YELLOW, COLOR_BLACK, BOLD, NO_BLINK		/* search highlight */
+     };
 #endif /* NO_COLOR_CURSES */
-int
-parse_config (void)
+     int
+       parse_config (void)
 {
   char config_file_name[256], *home = 0;
   char line[256];
@@ -409,6 +410,21 @@ parse_line (char *line)
 	    return 1;
 	}
     }
+  else if (!strncmp (temp, "VERBOSE", 7))
+    {
+      temp = strtok (NULL, "=");
+      if (temp)
+	{
+	  if (!(temp = str_toupper (skip_whitespace (temp))))
+	    return 1;
+	  if (!strncmp (temp, "TRUE", 4))
+	    verbose = 1;
+	  else if (!strncmp (temp, "FALSE", 5))
+	    verbose = 0;
+	  else
+	    return 1;
+	}
+    }
   else if (!strncmp (temp, "QUIT-CONFIRMATION", 17))
     {
       temp = strtok (NULL, "=");
