@@ -5,29 +5,29 @@
 
 #include "common_includes.h"
 
-RCSID(PKG_VER "$Id: pinfo.c,v 1.34 2000/01/27 11:38:15 misiek Exp $")
+RCSID (PKG_VER "$Id: pinfo.c,v 1.36 2000/08/27 19:09:48 pborys Exp $")
 
 #ifdef HAVE_GETOPT_LONG
 #include <getopt.h>
 #endif
 
-char *version = VERSION;
-int DontHandleWithoutTagTable = 0;
+     char *version = VERSION;
+     int DontHandleWithoutTagTable = 0;
 
-char *curfile = 0;		/* currently viewed filename */
+     char *curfile = 0;		/* currently viewed filename */
 
-char *pinfo_start_node = 0;	/* node specified by --node option */
+     char *pinfo_start_node = 0;	/* node specified by --node option */
 
-void strip_file_from_info_suffix (char *file);	/* 
-						 * strip `.info' suffix from
-						 * "file" 
-						 */
-char *addinfosuffix (char *file);	/* add `.info' suffix to "file" */
+     void strip_file_from_info_suffix (char *file);	/* 
+							 * strip `.info' suffix from
+							 * "file" 
+							 */
+     char *addinfosuffix (char *file);	/* add `.info' suffix to "file" */
 
-void checksu ();		/* protect against bad, bad macros */
+     void checksu ();		/* protect against bad, bad macros */
 
-int
-main (int argc, char *argv[])
+     int
+       main (int argc, char *argv[])
 {
   int filenotfound = 0;
   char filename[256];
@@ -62,7 +62,8 @@ main (int argc, char *argv[])
   searchagain.search = 0;
   initlocale ();
   inithistory ();
-  printf ("Przemek's Info Viewer v%s\n", version);
+  if (verbose)
+    printf ("Przemek's Info Viewer v%s\n", version);
   initpaths ();
   parse_config ();		/* read config information */
   checksu ();
@@ -76,7 +77,8 @@ main (int argc, char *argv[])
   if (strlen (argv[0]) >= 3)
     if (strstr (argv[0], "man") != NULL)	/* handle any 'man' alias to 'pinfo' */
       {
-	printf (_ ("Looking for man page...\n"));
+	if (verbose)
+	  printf (_ ("Looking for man page...\n"));
 	strcpy (filename, "");
 	for (i = 1; i < argc; i++)	/* pass all arguments to the `man' 
 					   command (manhandler calls `man') */
@@ -137,7 +139,8 @@ main (int argc, char *argv[])
 	case 'v':
 	  exit (0);
 	case 'm':
-	  printf (_ ("Looking for man page...\n"));
+	  if (verbose)
+	    printf (_ ("Looking for man page...\n"));
 	  strcpy (filename, "");
 	  for (i = optind; i < argc; i++)
 	    {
@@ -247,10 +250,12 @@ main (int argc, char *argv[])
   else
     /* ...otherwise try to create one */
     {
-      printf (_ ("Warning: tag table not found...\n"));
+      if (verbose)
+	printf (_ ("Warning: tag table not found...\n"));
       if (!DontHandleWithoutTagTable)
 	{
-	  printf (_ ("Trying to create alternate tag table...\n"));
+	  if (verbose)
+	    printf (_ ("Trying to create alternate tag table...\n"));
 	  create_tag_table (id);
 	  if (TagTableEntries < 1)	/* if there weren't found any info
 					   entries */
@@ -285,7 +290,7 @@ main (int argc, char *argv[])
 
       if (!filenotfound)	/* handle goto/link where no file was found 
 				   -- see bellow */
-	addinfohistory (curfile, tag_table[tag_table_pos].nodename, -1, -1);
+	addinfohistory (curfile, tag_table[tag_table_pos].nodename, -1, -1, -1);
       else
 	filenotfound = 0;
       work_return_value = work (&message, &type, &lines, id, tag_table_pos);
@@ -464,8 +469,11 @@ checksu ()
       grwd = getgrnam (safe_group);
       if (!grwd)
 	{
-	  printf (_ ("Security warning: Unable to get GID of group called: %s\n"), safe_group);
-	  sleep (1);
+	  if (verbose)
+	    {
+	      printf (_ ("Security warning: Unable to get GID of group called: %s\n"), safe_group);
+	      sleep (1);
+	    }
 	}
       else
 	{
@@ -481,8 +489,11 @@ checksu ()
       pswd = getpwnam (safe_user);
       if (!pswd)
 	{
-	  printf (_ ("Security warning: Unable to get UID of user called: %s\n"), safe_user);
-	  sleep (1);
+	  if (verbose)
+	    {
+	      printf (_ ("Security warning: Unable to get UID of user called: %s\n"), safe_user);
+	      sleep (1);
+	    }
 	}
       else
 	{
