@@ -1,16 +1,16 @@
 #include "common_includes.h"
 
-RCSID("$Id: utils.c,v 1.34 2000/01/27 11:38:16 misiek Exp $")
+RCSID ("$Id: utils.c,v 1.34 2000/01/27 11:38:16 misiek Exp $")
 
 #include <regex.h>
 #include <ctype.h>
 
-char *safe_user = "nobody";
-char *safe_group = "nobody";
+     char *safe_user = "nobody";
+     char *safe_group = "nobody";
 
 #ifndef HAVE_CURS_SET
-void
-curs_set (int a)
+     void
+       curs_set (int a)
 {
 }
 #endif
@@ -282,7 +282,7 @@ int
 gettagtablepos_search_internal (char *node, int left, int right)
 {
   int thispos = left + ((right - left) >> 1);	/* left+(right-left)/2 */
-  int compare_result = compare_tag_table_string (tag_table[thispos].nodename, node);
+  int compare_result = /*compare_tag_table_string*/ strcmp (tag_table[thispos].nodename, node);
   if (compare_result == 0)
     return thispos;
   else
