@@ -1,12 +1,12 @@
 
 #include "common_includes.h"
 
-RCSID("$Id: video.c,v 1.17 2000/01/27 11:38:16 misiek Exp $")
+RCSID ("$Id: video.c,v 1.17 2000/01/27 11:38:16 misiek Exp $")
 
-void info_add_highlights (int pos, int cursor, long lines, char **message);
+     void info_add_highlights (int pos, int cursor, long lines, char **message);
 
-void
-substitutestr (char *src, char *dest, char *from, char *to)
+     void
+       substitutestr (char *src, char *dest, char *from, char *to)
 /*
  * Utility for substituting strings in given string.
  * Used for internationalization of info headers.
@@ -57,6 +57,7 @@ showscreen (char **message, char *type, long lines, long pos, long cursor)
 #ifdef HAVE_BKGDSET
   bkgdset (' ' | normal);
 #endif
+  attrset(normal);
   for (i = pos; (i < lines) && (i < pos + maxy - 2); i++)
     {
       int tmp = strlen (message[i]) - 1;
